@@ -4,7 +4,6 @@ import os
 
 import notrequests
 
-from . import rsrc
 from . import utils
 
 
@@ -12,17 +11,16 @@ CODEBASE_API_URL = 'https://api3.codebasehq.com'
 logger = logging.getLogger(__file__)
 
 
-class Client(object):
+class _Client(object):
     """Codebase API client class."""
     def __init__(self, (username, key)):
-        self.projects = rsrc._ProjectController(_client=self)
         self.auth = (username, key)
         self.base_url = CODEBASE_API_URL
 
         if not self.base_url.endswith('/'):
             self.base_url += '/'
 
-    def _api_method(self, method, path, params=None, json=None):
+    def _api_method(self, method, path, params=None, json=None, files=None):
         url = self.base_url + path
         headers = {'Accept': 'application/json'}
 
@@ -34,6 +32,7 @@ class Client(object):
             params=params,
             headers=headers,
             json=json,
+            files=files,
             timeout=30,
         )
 
@@ -46,8 +45,11 @@ class Client(object):
 
         return response
 
-    def _api_post(self, path, params=None, json=None):
-        return self._api_method('POST', path, params=params, json=json)
+    def _api_put(self, path, params=None, json=None, files=None):
+        return self._api_method('PUT', path, params=params, json=json, files=files)
+
+    def _api_post(self, path, params=None, json=None, files=None):
+        return self._api_method('POST', path, params=params, json=json, files=files)
 
     def _api_get(self, path, params=None):
         return self._api_method('GET', path, params=params)
@@ -249,6 +251,84 @@ class Client(object):
 
         return data
 
+    def get_milestones(self, project):
+        """Get the milestones for a project.
+
+        :param project: permalink for a project
+        :type project: str
+        :rtype: generator
+        """
+        path = '%s/milestones' % (project,)
+
+        # Seems to be unpaginated.
+        response = self._api_get(path)
+        data = response.json()
+
+        for obj in data:
+            yield obj['ticketing_milestone']
+
+    def create_milestone(self, project, name, deadline=None, description=None,
+            estimated_time=None, parent_id=None,
+            responsible_user_id=None, start_at=None, status=None):
+        """Create a new milestone.
+
+        See the API documentation on `milestones`_ for details.
+
+        .. note:: The Codebase API allows multiple milestones to have the same
+            name, and this method does not check for duplicates.
+
+        .. _milestones: https://support.codebasehq.com/kb/tickets-and-milestones/milestones
+
+        :param project: permalink for a project
+        :param name: new milestone's name
+        :param deadline: the date of this milestone's deadline
+        :param description: a long description for the new milestone
+        :param estimated_time: the estimated time for the milestone
+        :param parent_id: the ID of this milestone's parent milestone
+        :param responsible_user_id: the id of the user responsible for the milestone
+        :param start_at: the date this milestone begins
+        :param status: the milestone status. One of "active", "completed" or "cancelled"
+        :type project: str
+        :type name: str
+        :type deadline: str|datetime.date
+        :type description: str
+        :type estimated_time: int
+        :type parent_id: str
+        :type responsible_user_id: str
+        :type start_at: str|datetime.date
+        :type status: str
+        :rtype: dict
+        """
+        path = '%s/milestones' % (project,)
+
+        milestone_data = utils.build_milestone_payload(deadline=deadline,
+            description=description, estimated_time=estimated_time, name=name,
+            parent_id=parent_id, responsible_user_id=responsible_user_id,
+            start_at=start_at, status=status)
+
+        payload = {'ticketing_milestone': milestone_data}
+        response = self._api_post(path, json=payload)
+        data = response.json()
+
+        return data
+
+    def update_milestone(self, project, milestone_id, deadline=None,
+            description=None, estimated_time=None, name=None, parent_id=None,
+            responsible_user_id=None, start_at=None, status=None):
+        """Update an existing milestone."""
+        path = '%s/milestones/%s' % (project, milestone_id)
+
+        milestone_data = utils.build_milestone_payload(deadline=deadline,
+            description=description, estimated_time=estimated_time, name=name,
+            parent_id=parent_id, responsible_user_id=responsible_user_id,
+            start_at=start_at, status=status)
+
+        payload = {'ticketing_milestone': milestone_data}
+        response = self._api_put(path, json=payload)
+        data = response.json()
+
+        return data
+
     def get_tickets(self, project, assignee=None, status=None, category=None,
             type=None, priority=None, milestone=None):
         """Get all tickets on a project, or search for tickets.
@@ -307,7 +387,7 @@ class Client(object):
             'ticket': {
                 'summary': summary,
                 'ticket_type': type,
-                'reporter_id': reported_id,
+                'reporter_id': reporter_id,
                 'assignee_id': assignee_id,
                 'category_id': category_id,
                 'priority_id': priority_id,
@@ -371,6 +451,34 @@ class Client(object):
 
         return data
 
+    def upload_files(self, files):
+        """Upload files.
+
+        Each file in the list can be one of:
+
+        * A file-like object open for reading.
+        * A pair of (filename, file-like object).
+        * A pair of (filename, byte-string).
+
+        Returns a generator of upload info dictionaries. The 'identifier' key
+        for an uploaded file can be used in the `upload_tokens` argument when
+        creating a ticket or note.
+
+        :param files: list of files to upload
+        :type files: list
+        :rtype: generator
+        """
+        # https://support.codebasehq.com/kb/uploading-files
+
+        path = 'uploads'
+        field_name = 'files[]'
+        files_data = [(field_name, obj) for obj in files]
+        response = self._api_post(path, files=files_data)
+        data = response.json()
+
+        for obj in data:
+            yield obj['upload']
+
     def get_ticket_statuses(self, project):
         """Get all status choices in a project.
 
@@ -524,6 +632,100 @@ class Client(object):
         return new_client_with_secrets_from_filename(cls, filename)
 
 
+class Client(_Client):
+    """Codebase API client class that allows some ticket properties to be
+    referenced by name instead of the object ID.
+    """
+    def __init__(self, *args, **kwargs):
+        super(Client, self).__init__(*args, **kwargs)
+        self.reset_cache()
+
+    def reset_cache(self):
+        self._cache = {}
+
+    def use_cache(self, project, kwargs):
+        """For some ticket classifiers (what Codebase calls organisational
+        objects, such as status, category, etc.), return a new dict with the
+        classifier's name replaced by the classifier's ID.
+
+        This works by making an API call to get ticket statuses, etc. and
+        caching the results for quick lookup. The cache can be cleared with
+        Client.reset_cache().
+        """
+        # We take an argument like 'category', lookup a value and reassign it
+        # to 'category_id'.
+        classifiers = {
+            'assignee': self.cache_assignee,
+            'category': self.cache_category,
+            'milestone': self.cache_milestone,
+            'priority': self.cache_priority,
+            'status': self.cache_status,
+        }
+
+        # Make a copy of the original, don't mutate the dict we were given.
+        kwargs = dict(kwargs)
+
+        # Could use a defaultdict or something. Anyway, setup project's cache.
+        if project not in self._cache:
+            self._cache[project] = {}
+
+        for name in classifiers:
+            if name in kwargs:
+                # Check if we have cached the results for this classifier. If
+                # not, then call a method like 'Client.cache_category()'
+                # and assign the result to the cache. N.B. The cache is
+                # per-project.
+                if name not in self._cache[project]:
+                    self._cache[project][name] = classifiers[name](project)
+
+                # OK. We've cached this classifier's stuff.
+                value = kwargs[name]
+                value = self._cache[project][name].get(value, value)
+
+                # Now set the '_id' version and unset the original argument.
+                kwargs[name + '_id'] = value
+                del kwargs[name]
+
+        return kwargs
+
+    def cache_assignee(self, project):
+        result = {}
+        users = self.get_project_users(project)
+
+        # We allow an assignee to be specified by the username or any of their
+        # email addresses.
+        for user in users:
+            user_id = user['id']
+            result[user['username']] = user_id
+
+            for email in user['email_addresses']:
+                result[email] = user_id
+
+        return result
+
+    def cache_category(self, project):
+        return {obj['name']: obj['id'] for obj in self.get_ticket_categories(project)}
+
+    def cache_milestone(self, project):
+        return {obj['name']: obj['id'] for obj in self.get_milestones(project)}
+
+    def cache_priority(self, project):
+        return {obj['name']: obj['id'] for obj in self.get_ticket_priorities(project)}
+
+    def cache_status(self, project):
+        return {obj['name']: obj['id'] for obj in self.get_ticket_statuses(project)}
+
+    def create_ticket(self, project, **kwargs):
+        kwargs = self.use_cache(project, kwargs)
+
+        return super(Client, self).create_ticket(project, **kwargs)
+
+    def create_ticket_note(self, project, ticket_id, **kwargs):
+        kwargs = self.use_cache(project, kwargs)
+
+        return super(Client, self).create_ticket_note(project, ticket_id, **kwargs)
+
+
 def new_client_with_secrets_from_filename(cls, filename):
     """Returns a new instance of codebase.Client. The username / key are read
     from the filename which must be in INI format. A filename like '~/.secrets'
